"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const snippetParser_1 = require("./snippetParser");
const array_1 = require("./array");
const check = (s) => {
    const snippet = new snippetParser_1.SnippetParser().parse(s, true);
    console.log(s);
    console.log(snippet.toString());
    for (const p of snippet.placeholders) {
        const placeholderOffset = snippet.offset(p);
        const placeholderLen = snippet.fullLen(p);
        console.log({
            index: p.index,
            placeholderOffset,
            placeholderLen,
            final: p.isFinalTabstop,
        });
    }
    // console.log(inspect(snippet.children, { depth: null }));
    // console.log(serializeSnippet(s));
    // console.log(inspect(serializeSnippet(s), { depth: null }));
    // console.log(snippet.placeholders);
    // console.log(snippet.toTextmateString());
};
const serializeSnippet = (s) => {
    const snippet = new snippetParser_1.SnippetParser().parse(s, true);
    return snippet.children.map(serializeMarker);
};
const serializeMarker = (s) => {
    var _a, _b;
    if (s instanceof snippetParser_1.Text) {
        return { type: "text", value: s.value };
    }
    if (s instanceof snippetParser_1.Placeholder) {
        return {
            type: "placeholder",
            index: s.index,
            transform: (_a = s.transform) === null || _a === void 0 ? void 0 : _a.toTextmateString(),
            children: s.children.map(serializeMarker),
        };
    }
    if (s instanceof snippetParser_1.Choice) {
        return {
            type: "choice",
            options: s.options.map((t) => t.value),
        };
    }
    if (s instanceof snippetParser_1.Variable) {
        return {
            type: "variable",
            name: s.name,
            transform: (_b = s.transform) === null || _b === void 0 ? void 0 : _b.toTextmateString(),
        };
    }
    return { type: "other", value: s.toTextmateString() };
};
// To do multi select.
// cm.setSelections([{anchor: {line: 0, ch: 0}, head: {line: 0, ch: 1}}], 0);
// cm.replaceSelection(value, "around");
// Need to prevent other events from happening when in snippet state.
// In Snippet completion state?
//   Tab
//     If placeholder index != 0, Select next placeholder(s).
//     Otherwise, stop Snippet completion state.
// If cursor moved out of the placeholders, then stop snippet state.
console.log(check('"${1:my-third-party-library}": "${2:1.2.3}"'));
console.log(check('"$1": "$1${2:1.2.3}"'));
console.log(check("${1:another ${2:placeholder}}"));
console.log(check("${1|one,two,three|}"));
console.log(check("${name:default}"));
console.log(check("${TM_FILENAME/(.*)\\..+$/$1/}"));
class Snippet {
    constructor(editor, snippet, offset) {
        this.editor = editor;
        this.snippet = snippet;
        this.offset = offset;
        this.placeholderGroups = array_1.groupBy(snippet.placeholders, snippetParser_1.Placeholder.compareByIndex);
        this.placeholderGroupsIdx = -1;
        // editor.setBookmark();
    }
    initMarks() {
        if (this.placeholderMarks)
            return;
        this.placeholderMarks = new Map();
        this.markToPlaceholder = new WeakMap();
        for (const placeholder of this.snippet.placeholders) {
            const placeholderOffset = this.snippet.offset(placeholder);
            const from = this.editor.posFromIndex(this.offset + placeholderOffset);
            const to = this.editor.posFromIndex(this.offset + placeholderOffset + this.snippet.fullLen(placeholder));
            const className = placeholder.isFinalTabstop
                ? "cmw-snippet-final-tabstop"
                : "cmw-snippet-tabstop";
            const mark = this.editor.markText(from, to, {
                className,
                clearWhenEmpty: false,
                // New text will be part of the marker
                inclusiveLeft: true,
                inclusiveRight: true,
            });
            this.placeholderMarks.set(placeholder, mark);
            this.markToPlaceholder.set(mark, placeholder);
        }
        // To update the marker, we need to clear and mark again.
        // Add event listener to update the marks on change within them.
        this.editor.on("change", (cm, { from, to, text, origin }) => {
            if (origin === "+snippet")
                return;
            const marks = from.line === to.line && from.ch === to.ch
                ? cm.findMarksAt(from)
                : cm.findMarks(from, to);
            for (const mark of marks) {
            }
        });
    }
    move(fwd = false) {
        this.initMarks();
        // Transform placeholder text if necessary
        if (this.placeholderGroupsIdx >= 0) {
            const ops = [];
            for (const placeholder of this.placeholderGroups[this.placeholderGroupsIdx]) {
                const transform = placeholder.transform;
                if (!transform)
                    continue;
                const mark = this.placeholderMarks.get(placeholder);
                const range = mark.find();
                const currentValue = this.editor.getRange(range.from, range.to);
                ops.push(() => {
                    this.editor.replaceRange(transform.resolve(currentValue), range.from, range.to, "+snippet");
                });
            }
            if (ops.length > 0) {
                this.editor.operation(() => {
                    for (const op of ops) {
                        op();
                    }
                });
            }
        }
        let couldSkipThisPlaceholder = false;
        if (fwd === true &&
            this.placeholderGroupsIdx < this.placeholderGroups.length - 1) {
            this.placeholderGroupsIdx += 1;
            couldSkipThisPlaceholder = true;
        }
        else if (fwd === false && this.placeholderGroupsIdx > 0) {
            this.placeholderGroupsIdx -= 1;
            couldSkipThisPlaceholder = true;
        }
        else {
            // the selection of the current placeholder might
            // not be accurate any more -> simply restore it
        }
        const selections = [];
        const active = new Set();
        for (const placeholder of this.placeholderGroups[this.placeholderGroupsIdx]) {
            const mark = this.placeholderMarks.get(placeholder);
            const range = mark.find();
            selections.push({ anchor: range.from, head: range.to });
            // update marker class
            active.add(placeholder);
        }
        return selections;
    }
}
//# sourceMappingURL=check-parser.js.map